# Re-bastardized version of spam-o-matic from Fedora's mash
###############################################################################
#Programmer: Orcan Ogetbil    orcan@nbcs.rutgers.edu                          #
#Date: 07/19/2010                                                             #
#Filename: checkdep.py                                                        #
#                                                                             #
#                                                                             #
#       Copyright 2010 Orcan Ogetbil                                          #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
###############################################################################
#
# Library for checking package dependencies

import os
import shutil
from stat import *
import string
import sys
import tempfile
import re
import smtplib
import logging
import getpass

from yum.constants import *
from yum.misc import getCacheDir

import koji
# HAAACK
import imp
sys.modules['repoclosure'] = imp.load_source("repoclosure","/usr/bin/repoclosure")
import repoclosure
sys.path.append('/usr/share/createrepo/')
sys.modules['createrepo'] = imp.load_source("createrepo","/usr/share/createrepo/genpkgmetadata.py")
import createrepo

import rcommon

config = rcommon.loadConfig()

deps = {}

def generateConfig(disttype, arch, tmprepodir=""):
    (fd, conffile) = tempfile.mkstemp()
    relname = config.get("repositories", "distname")
    relname_nice = config.get("repositories", "distname_nice")
    relver = config.get("repositories", "distver")
    repodir_public = config.get("repositories", "repodir_public")
    repodir_private = config.get("repositories", "repodir_private")
    repos = config.get("repositories", "publishrepos").split()
    distroverpkg = config.get("repositories", "distroverpkg")
    confheader = """
[main]
debuglevel=2
logfile=/var/log/yum.log
pkgpolicy=newest
distroverpkg=%s
reposdir=/dev/null
cachedir=/var/tmp/yum
keepcache=0

[base]
name=%s - Base
baseurl=file://%s/%s/os/%s/

[update]
name=%s - Updates
baseurl=file://%s/%s/updates/%s/

[addons]
name=%s - Addons
baseurl=file://%s/%s/addons/%s/

[extras]
name=%s - Extras
baseurl=file://%s/%s/extras/%s/
""" %(distroverpkg, relname_nice, repodir_public, relver, arch, relname_nice, repodir_public, relver, arch, relname_nice, repodir_public, relver, arch, relname_nice, repodir_public, relver, arch)

    for repo in repos:
        confheader += """
[%s]
name=%s %s Tree
baseurl=file://%s/%s/%s/%s/
""" %(repo, relname_nice, repo, repodir_private, repo, relver, arch)
        if repo == disttype:
            break

    if tmprepodir != "":
        confheader += """
[candidates]
name=New packages
baseurl=file://%s

""" %(tmprepodir)
    
    my_logger.info("I generated a yum.conf file with the repositories that I will")
    my_logger.info("use to do the dependency checking.")
    my_logger.debug("---------------------------yum.conf------------------------------")
    my_logger.debug(confheader)
    my_logger.debug("-----------------------------------------------------------------")
    os.write(fd,confheader)
    os.close(fd)
    return conffile


def libmunge(match):
    if match.groups()[1].isdigit():
        return "%s%d" % (match.groups()[0],int(match.groups()[1])+1)
    else:
        return "%s%s" % (match.groups()[0],match.groups()[1])

def getSrcPkg(pkg):
    if pkg.arch == 'src':
      return pkg.name
    srpm = pkg.returnSimple('sourcerpm')
    if not srpm:
        return None
    srcpkg = string.join(srpm.split('-')[:-2],'-')
    return srcpkg

def printableReq(pkg, dep):
    (n, f, v) = dep
    req = '%s' % n
    if f:
        flag = LETTERFLAGS[f]
        req = '%s %s' % (req, flag)
    if v:
        req = '%s %s' % (req, v)
    return "%s requires %s" % (pkg, req,)

def assignBlame(resolver, dep):
    def __addpackages(sack):
        for package in sack.returnPackages():
            p = getSrcPkg(package)
    
    # Given a dep, find potential responsible parties

    list = []
    
    # The dep itself
    list.append(dep)

    # Something that provides the dep
    __addpackages(resolver.whatProvides(dep, None, None))

    # Libraries: check for variant in soname
    if re.match("lib.*\.so\.[0-9]+",dep):
        new = re.sub("(lib.*\.so\.)([0-9])+",libmunge,dep)
        __addpackages(resolver.whatProvides(new, None, None))
        libname = dep.split('.')[0]
        __addpackages(resolver.whatProvides(libname, None, None))

    return list

def generateSpam(pkgname, treename):

    package = deps[pkgname]

    for s in package.keys():
        subpackage = package[s]
        for arch in subpackage.keys():
            brokendeps = subpackage[arch]

    data = """

%s has broken dependencies in the %s tree:
""" % (pkgname, treename)

    for s in package.keys():
        subpackage = package[s]
        for arch in subpackage.keys():
            data = data + "On %s:\n" % (arch)
            brokendeps = subpackage[arch]
            for dep in brokendeps:
                data = data + "\t%s\n" % printableReq(dep[0],dep[1])

    data = data + "Please resolve this as soon as possible.\n\n"
    
    return data


def filterout(deps):
    file = open(config.get("repositories", "depcheck_ignorefile"), 'r')
    inputtext = file.readlines()
    file.close()
    ignorelist = []
    for line in range(len(inputtext)):
        if inputtext[line][0] != "#" and inputtext[line] != "\n":
            ignoreitem = inputtext[line].split()
            if len(ignoreitem) == 3:
                ignorelist.append(ignoreitem)
            else:
                my_logger.warning("Could not parse "+config.get("repositories", "depcheck_ignorefile")+", line "+ line+ ":")
                my_logger.warning("\t"+inputtext[line])
                # Bad! We need to quit
                return -1

    pkgs = deps.keys()
    for pkg in pkgs:
        remove = 0
        for (n, f, v) in deps[pkg]:
            if filterrules(ignorelist, pkg.name, n, f, v):
                remove = 1
        if remove == 1:
            deps.pop(pkg)
    return deps

def strToRegex(expression):
    r_string = string.replace(expression, ".", "\.")
    r_string = string.replace(r_string, "*", ".*")
    r_string = string.replace(r_string, "+", "\+")
    r_string = string.replace(r_string, "(", "\(")
    r_string = string.replace(r_string, ")", "\)")
    return r_string

def filterrules(ignorelist, name,requires,compare,version):
    for ignoreitem in range(len(ignorelist)):
        myname = ignorelist[ignoreitem][0]
        myrequires = ignorelist[ignoreitem][1]
        myversion = ignorelist[ignoreitem][2]

        if string.find(ignorelist[ignoreitem][0], "*") > -1:
            ignore_regex_name = re.compile(strToRegex(ignorelist[ignoreitem][0]))
            if ignore_regex_name.match(name):
                myname = name
        if string.find(ignorelist[ignoreitem][1], "*") > -1:
            ignore_regex_requires = re.compile(strToRegex(ignorelist[ignoreitem][1]))
            if ignore_regex_requires.match(requires):
                myrequires = requires
        # Version may not be given, in which case it is a None type object
        if string.find(ignorelist[ignoreitem][2], "*") > -1:
            ignore_regex_version = re.compile(strToRegex(ignorelist[ignoreitem][2]))
            if version == None:
                myversion = version
            elif ignore_regex_version.match(version):
                myversion = version

        if name == myname and requires == myrequires and version == myversion:
            return True
    return False

def createTmpRepo(packages, arch):
    hub = config.get('koji','hub')
    kojisession = koji.ClientSession(hub)
    clientcert = config.get('koji','clientcert')
    clientca = config.get('koji','clientca')
    serverca = config.get('koji','serverca')
    kojisession.ssl_login(clientcert, clientca, serverca)

    tempprefix = 'tmprepo-'+arch+'-'
    tempdir = tempfile.mkdtemp(prefix=tempprefix)
    my_logger.info("Populating temporary repo " + tempdir + " with candidate packages.")
    for package in packages:
        buildinfo = kojisession.getBuild(package)
        packagedir = config.get("koji", "pkgdir")+'/'+buildinfo['name']+'/'+buildinfo['version']+'/'+buildinfo['release']
        # First copy arch specific RPM's
        try:
            rpmfiles = os.listdir(packagedir+'/'+arch)
            for file in rpmfiles:
                if file.endswith('.rpm') and string.find(file, 'debuginfo') == -1:
                    my_logger.info("Copying " + file)
                    shutil.copy2(packagedir+"/"+arch+"/"+file,tempdir)
        except OSError, ex:
            my_logger.warning("Package "+ package+ " does not contain "+ arch + " RPM's.")
            my_logger.debug(str(type(ex))+str(ex))
        # Next copy noarch RPM's
        if os.path.isdir(packagedir+'/noarch'):
            try:
                rpmfiles = os.listdir(packagedir+'/noarch')
                for file in rpmfiles:
                    if file.endswith('.rpm'):
                        my_logger.info("Copying " + file)
                        shutil.copy2(packagedir+"/noarch/"+file,tempdir)
            except IOError, ex:
                my_logger.debug(str(type(ex))+str(ex))
    my_logger.info("Writing metadata")
    createrepo.main([tempdir])
    return tempdir

def doit(type,packages=[]):
    global my_logger
    
    theSuspect = config.get("report","user")+"-"+getpass.getuser()        
    my_logger = logging.getLogger(theSuspect)

    i_am_broken = False

    archs = config.get("repositories", "archs").split()
    for arch in archs:
        my_logger.info("Arch: " + arch)
        if packages != []:
            tmprepodir = createTmpRepo(packages, arch)
        else:
            tmprepodir = ""
        
        conffile = generateConfig(type, arch, tmprepodir)
        if not conffile:
            continue
        if arch == 'i386':
            carch = 'i686'
        elif arch == 'ppc':
            carch = 'ppc64'
        elif arch == 'sparc':
            carch = 'sparc64v'
        else:
            carch = arch
        my_logger.info("I will next calculate the dependency chains.")
        my = repoclosure.RepoClosure(config = conffile, arch = [carch])
        cachedir = getCacheDir(reuse=False)
        my.repos.setCacheDir(cachedir)
        my_logger.info("Processing metadata returned from repoclosure...")
        my.readMetadata()
        my_logger.info("Now calculate the dependencies... It might take a minute or two.")
        baddeps = my.getBrokenDeps(newest = True)
        baddeps=filterout(baddeps)
        if baddeps == -1:
            return ["baddeps", "", ""]
        pkgs = baddeps.keys()
        tmplist = [(x.returnSimple('name'), x) for x in pkgs]
        tmplist.sort()
        pkgs = [x for (key, x) in tmplist]
        if len(pkgs) > 0:
            i_am_broken = True
            my_logger.warning("Broken deps for %s" % (arch))
            my_logger.warning("----------------------------------------------------------")
        else:
            my_logger.info("We are clean. No broken dependencies.")
        for pkg in pkgs:
            srcpkg = getSrcPkg(pkg)


            if not deps.has_key(srcpkg):
                deps[srcpkg] = {}

            pkgid = "%s-%s" % (pkg.name, pkg.printVer())

            if not deps[srcpkg].has_key(pkgid):
                deps[srcpkg][pkgid] = {}

            broken = []
            for (n, f, v) in baddeps[pkg]:
                my_logger.warning("\t%s" % printableReq(pkg, (n, f, v)))
                blamelist = assignBlame(my, n)
                broken.append( (pkg, (n, f, v), blamelist) )
                addtobrokenlist = True

                deps[srcpkg][pkgid][arch] = broken

        if tmprepodir != "":
            my_logger.info("Clean up: removing the temporary repo dir: "+ tmprepodir)
            shutil.rmtree(tmprepodir, ignore_errors = True)
        os.unlink(conffile)
        my_logger.info("Clean up: removing the cachedir: "+ cachedir)
        shutil.rmtree(cachedir, ignore_errors = True)
        my_logger.info("Done!\n\n")

    pkglist = deps.keys()

    spamdata = ""
    for pkg in pkglist:
        spamdata += generateSpam(pkg, type)

    if i_am_broken == True:
        email_subject = config.get("repositories", "distname_nice")+" - Broken dependencies"
        email_body = spamdata
        return ["broken", email_subject, email_body]
    else:
        return ["healthy","",""]
