###############################################################################
#Programmer: Orcan Ogetbil    orcan@nbcs.rutgers.edu                          #
#Date: 07/22/2010                                                             #
#Filename: populatedb.py                                                      #
#                                                                             #
#                                                                             #
#       Copyright 2010 Orcan Ogetbil                                          #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
###############################################################################
#
# Library that populates our rpm database

import os
import string
import _mysql
import _mysql_exceptions
import logging
import getpass
from MySQLdb.constants import FIELD_TYPE
import koji

import rcommon

config = rcommon.loadConfig()
hub = config.get("koji", "hub")
kojisession = koji.ClientSession(hub)

db_host = config.get("rpm2phpdb", "host")
db_user = config.get("rpm2phpdb", "user")
db_pw   = config.get("rpm2phpdb", "password")
db_name = config.get("rpm2phpdb", "name")

relver = config.get("repositories", "distver")
relname = config.get("repositories", "distname")

repo_prefix = relname + relver + "-"

repos = config.get("repositories", "allrepos").split()
for i in range(len(repos)):
	repos[i] = repo_prefix + repos[i]


def populateDatabase(db, content):
        myquery = """insert into Packages
        (package_id, build_id, rpm_id, build_name, nvr    , Name  , Version, Rel    , Arch    , URL    , SRCRPM , DBGRPM , Vendor, BuiltBy, Category, Summary, Description, License, Date, Size) values
        (%s        , %s      , %s    , \"%s\"    , \"%s\" , \"%s\", \"%s\" , \"%s\" , \"%s\"  , \"%s\" , \"%s\" , \"%s\" ,\"%s\" , \"%s\" , \"%s\"  , \"%s\" , \"%s\"     , \"%s\" , %s  , %s  )
        """ %(content['package_id'], content['build_id'], content['rpm_id'], content['build_name'], content['nvr'], content['Name'], content['Version'], content['Rel'], content['Arch'], content['URL'], content['SRCRPM'], content['DBGRPM'], content['Vendor'], content['BuiltBy'], content['Category'], content['Summary'], content['Description'], content['License'], content['Date'], content['Size'])
        db.query(myquery)
        for dep in content['dep_info']:
            if dep['type'] == 0:
                deptype = "Requires"
            elif dep['type'] == 1:
                deptype = "Provides"
            elif dep['type'] == 2:
                deptype = "Obsoletes"
            elif dep['type'] == 3:
                deptype = "Conflicts"
            
            depquery = """ insert into %s
            (rpm_id, Resource, Flags, Version, build_id) values
            ( %s   , \"%s\"  , %s   , \"%s\" , %s)
            """ %(deptype, content['rpm_id'], dep['name'], dep['flags'], dep['version'], content['build_id'])
            try:
                db.query(depquery)
            except _mysql_exceptions.IntegrityError:
                pass

        for file in content['file_info']:
	    filename = string.replace(file['name'], "\\", "\\\\")
            filequery = """ insert into Files
            (rpm_id, Path  , Size, build_id) values
            (%s    , \"%s\",%s   , %s)
            """ %(content['rpm_id'], filename, file['size'], content['build_id'])
            # Maybe we have to use .encode('utf-8') on filenames. Not necessary for now.
            try: # Normally, this shouldn't fail.
                db.query(filequery)
            except _mysql_exceptions.IntegrityError:
	        # Here we can get "ProgrammingError"s due to weird file names. To ignore this, we can replace the above line with
		# except (_mysql_exceptions.IntegrityError, _mysql_exceptions.ProgrammingError):
                pass
            
        for dist in content['dist_info']:
            distquery = """ insert into Distribution
            (rpm_id, repo, build_id) values
            (%s    ,\"%s\", %s)
            """ %(content['rpm_id'], dist, content['build_id'])
            db.query(distquery)

	for entry in content['specchangelog']:
		entry['author'] = string.replace(entry['author'], '"', '\\\"').encode('utf-8')
		entry['text'] = string.replace(entry['text'], '"', '\\\"').encode('utf-8')
		specclogquery = """ insert into SpecChangeLogs
		(build_id, Date, Author, Text, rpm_id) values
		(%s      , %s  , "%s"  , "%s", %s)
		""" %(content['build_id'], entry['date_ts'], entry['author'], entry['text'], content['rpm_id'])
		db.query(specclogquery)

	if content['softwarechangelog']:
		content['softwarechangelog'] = string.replace(content['softwarechangelog'], "\\", "\\\\")
		content['softwarechangelog'] = string.replace(content['softwarechangelog'], '"', '\\\"')
		content['softwarechangelog'] = string.replace(content['softwarechangelog'], "'", "\\\'")
		try:
                    content['softwarechangelog'] = content['softwarechangelog'].encode('utf-8')
		except UnicodeDecodeError:
                    pass
		softwareclogquery = """ insert into SoftwareChangeLogs
                (build_id, Filename, Text) values
                (%s      , "%s"    , "%s")
                """ %(content['build_id'], content['softwarechangelogfile'], content['softwarechangelog'])
		#print softwareclogquery
		db.query(softwareclogquery)

def RemoveOld(db):
    db.query("select distinct build_id from Packages")
    res = db.store_result()
    dat = res.fetch_row(maxrows=0)
    build_ids_in_db = []
    for entry in dat:
        build_ids_in_db.append(entry[0])
    latestBuilds = []
    for repo in repos:
        latestBuilds += kojisession.getLatestBuilds(repo)
    build_ids_in_repo = []
    for build in latestBuilds:
        build_ids_in_repo.append(build['build_id'])
    for build_id in build_ids_in_db:
        if not build_id in build_ids_in_repo:
	    List_query = """ select nvr, arch from Packages where
	    build_id = %s
	    """ %(build_id)
	    db.query(List_query)
	    res = db.store_result()
	    rpms2remove = res.fetch_row(maxrows=0)
	    
            for table in ['Packages', 'Distribution', 'Files', 'Requires', 'Provides', 'Obsoletes', 'Conflicts', 'SpecChangeLogs', 'SoftwareChangeLogs']:
                Packages_query = """ delete from %s where
                build_id = %s
                """ %(table, build_id)
                db.query(Packages_query)
	    for nvr in rpms2remove:
	        my_logger.info("Package: " + nvr[0] + "." + nvr[1] + " removed from database.")
                                                    

def FetchRepo(db):
    db.query("select distinct build_id from Packages")
    res = db.store_result()
    dat = res.fetch_row(maxrows=0)
    build_ids_in_db = []
    for entry in dat:
        build_ids_in_db.append(entry[0])

    latestBuilds = []
    for repo in repos:
        latestBuilds += kojisession.getLatestBuilds(repo)
    for build in latestBuilds:
        build_id = build['build_id']
        tags = kojisession.listTags(build_id)
        dist_info = []
        for tag in tags:
            dist_info.append(tag['name'])
        
        if build_id in build_ids_in_db:
            tagquery = "select distinct repo from Distribution where build_id=" + str(build_id)
            db.query(tagquery)
            res = db.store_result()
            dat = res.fetch_row(maxrows=0)
            tags_in_db = []
            for entry in dat:
                tags_in_db.append(entry[0])
            for tag in dist_info:
                if not tag in tags_in_db:
                    build_rpms = kojisession.listBuildRPMs(build_id)
                    for rpm in build_rpms:
                        if rpm['arch'] != 'src' and not rpm['name'].endswith('-debuginfo'):
                            newtagquery = """ insert into Distribution
                            (rpm_id, repo, build_id) values
                            (%s    ,"%s", %s)
                            """ %(rpm['id'], tag, build_id)
                            my_logger.info("New tag: " + tag + " for package: " + rpm['nvr'] + "." + rpm['arch'])
                            db.query(newtagquery)
            for tag in tags_in_db: # Reverse check
                if not tag in dist_info:
                    build_rpms = kojisession.listBuildRPMs(build_id)
                    for rpm in build_rpms:
                        if rpm['arch'] != 'src' and not rpm['name'].endswith('-debuginfo'):
                            removetagquery = """ delete from Distribution where
                            rpm_id = %s and repo = "%s"
                            """ %(rpm['id'], tag)
                            my_logger.info("Tag: " + tag + " removed from package: " + rpm['nvr'] + "." + rpm['arch'])
                            db.query(removetagquery)
        else:
            build_ids_in_db.append(build_id)
            
            build_name = build['name']
            package_id = build['package_id']
            BuiltBy = build['owner_name']
            build_rpms = kojisession.listBuildRPMs(build_id)
            build_nvr = ""
	    pkgdir = config.get("koji", "pkgdir")
            build_dbg = None
	    build_sw_clog = None
            for rpm in build_rpms:
                if rpm['arch'] == 'src':
                    build_nvr = rpm['nvr']
                elif rpm['name'].endswith('-debuginfo'):
                    build_dbg = rpm['nvr']
            for rpm in build_rpms:
                if rpm['arch'] != 'src' and not rpm['name'].endswith('-debuginfo'):
                    rpminfo = {}
                    rpminfo['build_id'] = build_id
                    rpminfo['package_id'] = package_id
                    rpminfo['rpm_id'] = rpm['id']
                    rpminfo['build_name'] = build_name
                    rpminfo['BuiltBy'] = BuiltBy
                    rpminfo['dist_info'] = dist_info
                    rpminfo['nvr'] = rpm['nvr']
                    rpminfo['Name'] = rpm['name']
                    rpminfo['Version'] = rpm['version']
                    rpminfo['Rel'] = rpm['release']
                    rpminfo['Arch'] = rpm['arch']
                    rpminfo['Size'] = rpm['size']
                    rpminfo['Date'] = rpm['buildtime']
                    rpminfo['SRCRPM'] = build_nvr + '.src.rpm'
                    if build_dbg and rpm['arch'] != 'noarch':
                        rpminfo['DBGRPM'] = build_dbg
                    else:
                        rpminfo['DBGRPM'] = None
                    
                    headers = ['description', 'summary', 'group', 'vendor', 'url', 'license']
                    harray = kojisession.getRPMHeaders(rpmID=rpm['id'], headers=headers)
                    rpminfo['URL'] = harray['url']
                    rpminfo['Vendor'] = harray['vendor']
                    rpminfo['Category'] = harray['group']
                    rpminfo['Summary'] = string.replace(harray['summary'], '"', '\\\"').encode('utf-8')
                    rpminfo['Description'] = string.replace(harray['description'], '"', '\\\"').encode('utf-8')
                    rpminfo['License'] = harray['license']

                    rpminfo['dep_info'] = kojisession.getRPMDeps(rpm['id'])
                    rpminfo['file_info'] = kojisession.listRPMFiles(rpm['id'])
		    rpminfo['specchangelog'] = kojisession.getChangelogEntries(build_id)

		    rpm_filepath = pkgdir + "/" + rpminfo['build_name'] + "/" + build['version'] + "/" + build['release'] + "/" + rpm['arch'] + "/" + rpm['name'] + "-" + rpm['version'] + "-" + rpm['release'] + "." + rpm['arch'] + ".rpm"
		    rpminfo['softwarechangelog'] = ""
		    rpminfo['softwarechangelogfile'] = ""
		    if not build_sw_clog:
                        cloginfo = getSoftwareClog(rpm_filepath)
			if cloginfo:
                            #print cloginfo
			    rpminfo['softwarechangelogfile'] = cloginfo[0]
                            rpminfo['softwarechangelog'] = cloginfo[1]
		    if rpminfo['softwarechangelogfile']:
                        build_sw_clog = True

                    my_logger.info(rpminfo['nvr'] + "." + rpminfo['Arch'] + " with rpm_id " + str(rpm['id']) + " entered to the database.")
                    populateDatabase(db, rpminfo)

def getSoftwareClog(rpmfile):
    filelist_command = "rpm2cpio " + rpmfile  + " | cpio -t --quiet"
    a = os.popen(filelist_command)
    filelist = a.readlines()
    a.close()

    clogfile = None
    cloglist = ["changeLog", "Changes", "NEWS"]
    for file in filelist:
	if not "/usr/share/doc" in file:
	    continue
	for clogcandidate in cloglist:
            if string.lower(clogcandidate) == string.lower(file[string.rfind(file, "/")+1:]):
                clogfile = file[:-1]
		break

	if not clogfile:
	    for clogcandidate in cloglist:
                if string.lower(clogcandidate) in string.lower(file):
                    clogfile = file[:-1]
		    break

	if clogfile:
            break

    if clogfile:
        clog_command = "rpm2cpio " + rpmfile + " | cpio -i --to-stdout --quiet " + clogfile
	a = os.popen(clog_command)
	swclog = a.readlines()
	a.close()
	#print clogfile, string.join(swclog)
	return [clogfile[1:], string.join(swclog)]

    return False

def CleanDatabase(db):
    dropquery = "drop tables if exists Distribution, Requires, Provides, Obsoletes, Conflicts, Files, Packages, SpecChangeLogs, SoftwareChangeLogs"
    db.query(dropquery)
    my_logger.info("Tables removed from the database")

def CreateTables(db):
    Packages_query = """create table if not exists Packages (package_id int(11) not null,
    build_id int(11) not null, rpm_id int(11), primary key(rpm_id), build_name varchar(50) not null,
    nvr varchar(255) not null, Name varchar(50) not null,
    Version varchar(50) not null, Rel varchar(50) not null, Arch varchar(15) not null,
    URL varchar(255), SRCRPM varchar(255) not null,
    DBGRPM varchar(255), Vendor varchar(50), BuiltBy varchar(20) not null,
    Category varchar(255) not null, Summary varchar(255), Description text,
    License varchar(255), Date int(11) not null, Size int(11) not null)"""
    db.query(Packages_query)
    Files_query = """create table if not exists Files (rpm_id int(11), Path varchar(255),
    primary key(rpm_id, Path), Size int(11) not null, build_id int(11) not null)"""
    db.query(Files_query)
    Requires_query = """create table if not exists Requires (rpm_id int(11), Resource varchar(50),
    primary key(rpm_id, Resource), Flags int(31), Version varchar(20), build_id int(11) not null)"""
    db.query(Requires_query)
    Provides_query = """create table if not exists Provides (rpm_id int(11), Resource varchar(50),
    primary key(rpm_id, Resource), Flags int(31), Version varchar(20), build_id int(11) not null)"""
    db.query(Provides_query)
    Obsoletes_query = """create table if not exists Obsoletes (rpm_id int(11), Resource varchar(50),
    primary key(rpm_id, Resource), Flags int(31), Version varchar(20), build_id int(11) not null)"""
    db.query(Obsoletes_query)
    Conflicts_query = """create table if not exists Conflicts (rpm_id int(11), Resource varchar(50),
    primary key(rpm_id, Resource), Flags int(31), Version varchar(20), build_id int(11) not null)"""
    db.query(Conflicts_query)
    Distribution_query = """create table if not exists Distribution (rpm_id int(11), repo varchar(50),
    primary key(rpm_id, repo), build_id int(11) not null)"""
    db.query(Distribution_query)
    SpecChangeLogs_query = """create table if not exists SpecChangeLogs (ID int(15) AUTO_INCREMENT,
    build_id int(11), Date int(11), Author varchar(255), primary key(ID), Text text, rpm_id int(11))"""
    db.query(SpecChangeLogs_query)
    SoftwareChangeLogs_query = """create table if not exists SoftwareChangeLogs (build_id int(11),
    primary key(build_id), Filename varchar(255), Text text)"""
    db.query(SoftwareChangeLogs_query)
    
    my_logger.info("Tables Created (if they didn't exist)")



#if __name__ == '__main__':
def UpdateDB(clean=False, create=False, removeoldpkg=True):
	global my_logger
	theSuspect = config.get("report","user")+"-"+getpass.getuser()
	my_logger = logging.getLogger(theSuspect)
	
	my_conv = { FIELD_TYPE.LONG: int }
	db = _mysql.connect(db_host, db_user, db_pw, db_name, conv=my_conv)
	
	if clean:
		CleanDatabase(db)
	if create:
		CreateTables(db)
	if removeoldpkg:
		RemoveOld(db)
	FetchRepo(db)
	
	db.close()
