#!/usr/bin/env python
###############################################################################
#Programmer: Orcan Ogetbil    orcan@nbcs.rutgers.edu                          #
#Date: 07/22/2010                                                             #
#Filename: rcommon.py                                                         #
#                                                                             #
#                                                                             #
#       Copyright 2010 Orcan Ogetbil                                          #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
###############################################################################
#
# Common functions to use in our scripts

import os
import sys
import grp
import getpass
import inspect
import time
import ConfigParser

def loadConfig():
    config = ConfigParser.ConfigParser()
    config.read("/etc/rutgers-repotools.cfg")
    return config

def verifyUserGroup():
    gid = os.getgid()
    grpname = grp.getgrgid(gid)[0]
    username = getpass.getuser()
    config = loadConfig()
    theSuspect = config.get("report","user")+"-" + username
    if username == "root":
        print "Error: Please do not run this script as root."
        sys.exit(1)
    if not grpname in config.get("repositories", "groupowner").split(" "):
        print "Error: The user who runs this script must belong to one of the groups: " + config.get("repositories", "groupowner")
        sys.exit(1)


class AppHandler:
    def __init__(self):
        # what script is calling this function?
        self.callerfile = inspect.stack()[-1][1]
        self.callername = self.callerfile.split("/")[-1]
        self.lockfilename = "/var/lock/rutgers-repotools/" + self.callername
        self.start_time = time.time()

    def timeRun(self):
        return round(time.time() - self.start_time,3)

    def checkLock(self):
        if os.path.isfile(self.lockfilename):
            print "An instance of " + self.callername + " is already running. Please wait until it exits, or"
            print "if this is not the case, remove the lock file " + self.lockfilename
            sys.exit(1)

    def createLock(self):
        self.checkLock()
        open(self.lockfilename, "w").close()
        
    def removeLock(self):
        if os.path.isfile(self.lockfilename):
            os.remove(self.lockfilename)

    def exit(self, status=0):
        self.removeLock()
        sys.exit(status)
