###############################################################################
#Programmer: Orcan Ogetbil    orcan@nbcs.rutgers.edu                          #
#Date: 07/19/2010                                                             #
#Filename: rlogger.py                                                         #
#                                                                             #
#                                                                             #
#       Copyright 2010 Orcan Ogetbil                                          #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
###############################################################################
#
# The logging facility for our scripts

import sys
import logging
import logging.handlers
import getpass

import rcommon

class LogFile(object):
    """File-like object to log text using the `logging` module."""

    def __init__(self, logger, level=logging.DEBUG):
        self.logger = logger
        self.level = level
        
    def write(self, msg):
        if self.level == logging.ERROR:
            self.logger.error(msg)
        elif self.level == logging.DEBUG:
            self.logger.debug(msg)
        else:
            self.logger.info(msg)
        
    def flush(self):
        for handler in self.logger.handlers:
            handler.flush()

                                                    

def init(FILENAME, level=logging.INFO, quiet=False):
    config = rcommon.loadConfig()
    theSuspect = config.get("report","user")+"-"+getpass.getuser()
    my_logger = logging.getLogger(theSuspect)
    my_logger.propagate=False
    my_logger.setLevel(logging.DEBUG)
    
    # We have two handlers. One to stdout, one to the log file.
    if quiet == False:
        shandler = logging.StreamHandler()
        my_logger.addHandler(shandler)
        #shandler.setFormatter(formatter) # keep this line commented out for now
        shandler.setLevel(level)

    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")

    if FILENAME != "":
        rfhandler = logging.handlers.RotatingFileHandler(FILENAME, maxBytes=int(config.get("logs", "maxBytes")), backupCount=int(config.get("logs", "backupCount")))
        my_logger.addHandler(rfhandler)
        rfhandler.setFormatter(formatter)
        rfhandler.setLevel(logging.DEBUG) # make the logfile always most verbose

    badlogger = logging.getLogger("yum.verbose.repoclosure")
    for handler in badlogger.handlers:
        badlogger.removeHandler(handler)

    # Also direct the stdout to the log
    sys.stdout = LogFile(my_logger, logging.DEBUG)
    sys.stderr = LogFile(my_logger, logging.ERROR)

